"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* tslint:disable:no-conditional-assignment ordered-imports */
var codemirror_1 = __importDefault(require("codemirror"));
require("codemirror/mode/xml/xml");
function getTagRegexp(tagName, anchored) {
    return new RegExp((anchored ? "^" : "") + "<\/\s*" + tagName + "\s*>", "i");
}
var REG_IN_TAG = /^([\S]+) (.*)/;
var REG_TAG = /\btag\b/;
var REG_ATTR = /\s+(\S+)\s*=\s*('([^']*)'|"([^"]*)")/g;
function createTag(text) {
    var _a = __read(REG_IN_TAG.exec(text) || [], 3), name = _a[1], content = _a[2];
    if (!name) {
        return null;
    }
    var attributes = {};
    var parts;
    while ((parts = REG_ATTR.exec(content)) !== null) {
        attributes[parts[1]] = parts[3] || parts[4];
    }
    return {
        name: name,
        attributes: attributes,
    };
}
function defaultDecideMode(ignored) {
    return null;
}
exports.defaultDecideMode = defaultDecideMode;
var CDATA_OPEN = '<![CDATA[';
var CDATA_CLOSE = ']]>';
codemirror_1.default.defineMode("xmlmixed", function (config, parserConfig) {
    if (parserConfig === void 0) { parserConfig = {}; }
    var xmlMode = codemirror_1.default.getMode(config, __assign({ name: "xml" }, parserConfig.xmlOptions));
    var decideMode = parserConfig.decideMode || defaultDecideMode;
    function xml(stream, state) {
        var style = state.xmlMode.token(stream, state.xmlState);
        var tag = style && REG_TAG.test(style);
        var tagName = state.xmlState.tagName;
        if (tag && !/[<>\s\/]/.test(stream.current()) && tagName) { // 标签的开头
            state.inTag = tagName + " ";
        }
        else if (state.inTag && tag && />$/.test(stream.current())) { // 标签的结束
            var tagObj = createTag(state.inTag);
            if (tagObj) {
                var mode = decideMode(tagObj);
                var modeObj = mode && codemirror_1.default.getMode(config, mode);
                if (modeObj) {
                    tagObj.mode = modeObj;
                    state.tag = tagObj;
                    state.token = xmlMixed;
                    state.localMode = modeObj;
                    state.localState = codemirror_1.default.startState(modeObj, xmlMode.indent(state.xmlState, "", ""));
                }
            }
            state.inTag = null;
        }
        else if (state.inTag) { // 标签的中间
            state.inTag += stream.current();
            if (stream.eol())
                state.inTag += " ";
        }
        return style;
    }
    function xmlMixed(stream, state) {
        var _a, _b;
        state.cdataEaten = false;
        if (state.cdata && stream.match(CDATA_CLOSE, true)) {
            state.cdata = false;
            state.cdataEaten = true;
            return 'cdata close';
        }
        if (!state.cdata) {
            if (stream.match(CDATA_OPEN, true)) {
                state.cdata = true;
                state.cdataEaten = true;
                return 'cdata open';
            }
            if (stream.match(getTagRegexp((_a = state.tag) === null || _a === void 0 ? void 0 : _a.name, true), false)) {
                state.tag = null;
                state.token = xml;
                state.localMode = state.localState = null;
                return null;
            }
        }
        var idxCdOpen = state.cdata ? -1 : stream.string.indexOf(CDATA_OPEN, stream.start);
        var idxCdClose = state.cdata ? stream.string.indexOf(CDATA_CLOSE, stream.start) : -1;
        var idxExit = state.cdata ? -1 : stream.string.search(getTagRegexp((_b = state.tag) === null || _b === void 0 ? void 0 : _b.name, false));
        if (idxExit < stream.start) {
            idxExit = -1;
        }
        var style = state.localMode.token(stream, state.localState);
        var backUp = false;
        if (idxCdOpen !== -1 && stream.pos > idxCdOpen) {
            stream.backUp(stream.pos - idxCdOpen);
            backUp = true;
        }
        if (idxCdClose !== -1 && stream.pos > idxCdClose) {
            stream.backUp(stream.pos - idxCdClose);
            backUp = true;
        }
        if (idxExit !== -1 && stream.pos > idxExit) {
            stream.backUp(stream.pos - idxExit);
            backUp = true;
        }
        if (backUp && stream.start === stream.pos) {
            return null;
        }
        return style;
    }
    // noinspection JSUnusedGlobalSymbols
    var theMode = {
        startState: function () {
            var state = codemirror_1.default.startState(xmlMode);
            return {
                token: xml,
                inTag: null,
                tag: null,
                localMode: null,
                localState: null,
                xmlMode: xmlMode,
                xmlState: state,
                cdata: false,
                cdataEaten: false,
            };
        },
        copyState: function (state) {
            var local = null;
            if (state.localState && state.localMode) {
                local = codemirror_1.default.copyState(state.localMode, state.localState);
            }
            var xmlState = null;
            if (state.xmlState) {
                xmlState = codemirror_1.default.copyState(xmlMode, state.xmlState);
            }
            return {
                token: state.token,
                inTag: state.inTag,
                tag: state.tag,
                localMode: state.localMode,
                localState: local,
                xmlMode: state.xmlMode,
                xmlState: xmlState,
                cdata: state.cdata,
                cdataEaten: state.cdataEaten,
            };
        },
        token: function (stream, state) {
            return state.token(stream, state);
        },
        indent: function (state, textAfter, line) {
            if (!state.localMode || /^\s*<\//.test(textAfter)) {
                return xmlMode.indent(state.xmlState, textAfter, line);
            }
            else if (state.localMode.indent) {
                return state.localMode.indent(state.localState, textAfter, line);
            }
            else {
                return codemirror_1.default.Pass;
            }
        },
        innerMode: function (state) {
            if (state.tag && state.cdataEaten) {
                return {
                    state: state,
                    mode: theMode,
                };
            }
            return {
                state: state.localState || state.xmlState,
                mode: state.localMode || xmlMode,
            };
        },
        closeBrackets: {
            pairs: '',
        },
    };
    return theMode;
});
codemirror_1.default.registerGlobalHelper('fold', 'cdata', function () { return true; }, function (cm, start) {
    var ch = start.ch, line = start.line;
    var text = cm.getLine(line);
    var idx = text.indexOf('<![CDATA[', ch);
    if (idx > -1) {
        var type = cm.getTokenTypeAt(codemirror_1.default.Pos(line, idx + 1));
        if (type && type.indexOf('cdata open') !== -1) {
            var from = codemirror_1.default.Pos(line, idx + 9);
            var end = text.indexOf(']]>', idx + 9);
            if (end > -1) {
                return {
                    from: from,
                    to: codemirror_1.default.Pos(line, end),
                };
            }
            else {
                for (var l = line + 1; l <= cm.lastLine(); ++l) {
                    text = cm.getLine(l);
                    end = text.indexOf(']]>');
                    if (end > -1) {
                        return {
                            from: from,
                            to: codemirror_1.default.Pos(l, end),
                        };
                    }
                }
            }
        }
    }
    return undefined;
});
//# sourceMappingURL=index.js.map